import { SemanticMark } from "./semantic-mark";

import { GrammarMarkType } from "@visactor/vgrammar-core";

import { isArray } from "@visactor/vutils";

import { field as getFieldAccessor } from "@visactor/vgrammar-util";

import { PlotMakType } from "./enums";

export class ImageSemanticMark extends SemanticMark {
    constructor(id) {
        super(PlotMakType.image, id);
    }
    setMarkType() {
        return GrammarMarkType.image;
    }
    parseScaleOfEncodeX(option) {
        const res = super.parseScaleOfEncodeX(option);
        return res.type = "point", res;
    }
    parseScaleByEncode(channel, option) {
        return "src" === channel ? {
            type: "ordinal",
            id: this.getScaleId("src"),
            domain: {
                data: this.getDataIdOfMain(),
                field: option
            }
        } : this.parseScaleOfCommonEncode(channel, option);
    }
    convertMarkEncode(encode) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        const markEncoder = this.convertSimpleMarkEncode(encode), scaleXId = this.getScaleId("x"), scaleYId = this.getScaleId("y"), res = {
            image: markEncoder.src
        };
        if (markEncoder.src && (res.image = markEncoder.src), isArray(null === (_a = markEncoder.x) || void 0 === _a ? void 0 : _a.field)) {
            const xAccessor = getFieldAccessor(markEncoder.x.field[0]), x1Accessor = getFieldAccessor(markEncoder.x.field[1]);
            res.x = (datum, el, params) => {
                const scale = params[scaleXId];
                return Math.min(scale.scale(xAccessor(datum)), scale.scale(x1Accessor(datum)));
            }, res.width = (datum, el, params) => {
                const scale = params[scaleXId];
                return Math.abs(scale.scale(xAccessor(datum)) - scale.scale(x1Accessor(datum)));
            };
        } else {
            const width = null !== (_c = null === (_b = this.spec.style) || void 0 === _b ? void 0 : _b.width) && void 0 !== _c ? _c : ImageSemanticMark.defaultSpec.width, xAccessor = getFieldAccessor(markEncoder.x.field);
            res.x = (datum, el, params) => params[scaleXId].scale(xAccessor(datum)) - width / 2, 
            res.width = width;
        }
        if (isArray(null === (_d = markEncoder.y) || void 0 === _d ? void 0 : _d.field)) {
            const yAccessor = getFieldAccessor(markEncoder.y.field[0]), y1Accessor = getFieldAccessor(markEncoder.y.field[1]);
            res.y = (datum, el, params) => {
                const scale = params[scaleYId];
                return Math.min(scale.scale(yAccessor(datum)), scale.scale(y1Accessor(datum)));
            }, res.height = (datum, el, params) => {
                const scale = params[scaleYId];
                return Math.abs(scale.scale(yAccessor(datum)) - scale.scale(y1Accessor(datum)));
            };
        } else {
            const height = null !== (_f = null === (_e = this.spec.style) || void 0 === _e ? void 0 : _e.height) && void 0 !== _f ? _f : ImageSemanticMark.defaultSpec.height, yAccessor = getFieldAccessor(markEncoder.y.field);
            res.y = (datum, el, params) => params[scaleXId].scale(yAccessor(datum)) - height / 2, 
            res.height = height;
        }
        return markEncoder.stroke && (res.stroke = markEncoder.stroke), markEncoder.color || markEncoder.group ? res.fill = null !== (_g = markEncoder.color) && void 0 !== _g ? _g : markEncoder.group : res.fill = null !== (_j = null === (_h = this.spec.style) || void 0 === _h ? void 0 : _h.fill) && void 0 !== _j ? _j : null === (_k = this.getPalette()) || void 0 === _k ? void 0 : _k[0], 
        res;
    }
}

ImageSemanticMark.type = PlotMakType.image, ImageSemanticMark.defaultSpec = {
    width: 10,
    height: 10
};
//# sourceMappingURL=image.js.map